package com.jpattern.orm.session;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.jpattern.logger.ILogger;
import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.exception.OrmNotUniqueResultException;
import com.jpattern.orm.logger.OrmLogger;

/**
 * 
 * @author Francesco Cina
 *
 * 02/lug/2011
 */
public class PlainSqlPerformer implements SqlPerformer {

	private final ILogger logger = OrmLogger.getOrmLogger(this.getClass());
	private final SqlPerformerStrategy sqlPerformerTemplate;
	private int queryTimeout = 0;
	private int maxRows = 0;

	public PlainSqlPerformer(final SqlPerformerStrategy sqlPerformerTemplate) {
		this.sqlPerformerTemplate = sqlPerformerTemplate;
	}

	@Override
	public final void setMaxRows(final int maxRows) {
		this.logger.debug("setMaxRows", "max rows sets to " + maxRows);
		this.maxRows = maxRows;
	}

	@Override
	public final int getMaxRows() {
		return this.maxRows;
	}

	@Override
	public final void setQueryTimeout(final int queryTimeout) {
		this.logger.debug("setQueryTimeout", "queryTimeout sets to " + queryTimeout);
		this.queryTimeout = queryTimeout;
	}

	@Override
	public final int getQueryTimeout() {
		return this.queryTimeout;
	}

	@Override
	public final Integer queryForInt(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
		ResultSetReader<Integer> resultSetReader = new ResultSetReader<Integer>() {

			@Override
			public Integer read(final ResultSet resultSet) throws SQLException {
				if ( resultSet.next() ) {
					int result = resultSet.getInt(1);
					if (resultSet.next()) {
						throw new OrmNotUniqueResultException("The query execution returned a number of rows higher than 1");
					}
					return result;
				}
				throw new OrmNotUniqueResultException("The query execution has returned zero rows. One row was expected");
			}
		};
		return this.query(sql, resultSetReader, values);
	}

	@Override
	public final Long queryForLong(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
		ResultSetReader<Long> resultSetReader = new ResultSetReader<Long>() {

			@Override
			public Long read(final ResultSet resultSet) throws SQLException {
				if ( resultSet.next() ) {
					long result = resultSet.getLong(1) ;
					if (resultSet.next()) {
						throw new OrmNotUniqueResultException("The query execution returned a number of rows higher than 1");
					}
					return result;
				}
				throw new OrmNotUniqueResultException("The query execution has returned zero rows. One row was expected");
			}
		};
		return this.query(sql, resultSetReader, values);
	}

	@Override
	public final Double queryForDouble(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
		ResultSetReader<Double> resultSetReader = new ResultSetReader<Double>() {

			@Override
			public Double read(final ResultSet resultSet) throws SQLException {
				if ( resultSet.next() ) {
					double result = resultSet.getDouble(1) ;
					if (resultSet.next()) {
						throw new OrmNotUniqueResultException("The query execution returned a number of rows higher than 1");
					}
					return result;
				}
				throw new OrmNotUniqueResultException("The query execution has returned zero rows. One row was expected");
			}
		};
		return this.query(sql, resultSetReader, values);
	}

	@Override
	public final Float queryForFloat(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
		ResultSetReader<Float> resultSetReader = new ResultSetReader<Float>() {

			@Override
			public Float read(final ResultSet resultSet) throws SQLException {
				if ( resultSet.next() ) {
					float result = resultSet.getFloat(1) ;
					if (resultSet.next()) {
						throw new OrmNotUniqueResultException("The query execution returned a number of rows higher than 1");
					}
					return result;
				}
				throw new OrmNotUniqueResultException("The query execution has returned zero rows. One row was expected");
			}
		};
		return this.query(sql, resultSetReader, values);
	}

	@Override
	public final String queryForString(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
		ResultSetReader<String> resultSetReader = new ResultSetReader<String>() {

			@Override
			public String read(final ResultSet resultSet) throws SQLException {
				if ( resultSet.next() ) {
					String result = resultSet.getString(1) ;
					if (resultSet.next()) {
						throw new OrmNotUniqueResultException("The query execution returned a number of rows higher than 1");
					}
					return result;
				}
				throw new OrmNotUniqueResultException("The query execution has returned zero rows. One row was expected");
			}
		};
		return this.query(sql, resultSetReader, values);
	}

	@Override
	public final Boolean queryForBoolean(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
		ResultSetReader<Boolean> resultSetReader = new ResultSetReader<Boolean>() {

			@Override
			public Boolean read(final ResultSet resultSet) throws SQLException {
				if ( resultSet.next() ) {
					boolean result = resultSet.getBoolean(1) ;
					if (resultSet.next()) {
						throw new OrmNotUniqueResultException("The query execution returned a number of rows higher than 1");
					}
					return result;
				}
				throw new OrmNotUniqueResultException("The query execution has returned zero rows. One row was expected");
			}
		};
		return this.query(sql, resultSetReader, values);
	}

	@Override
	public final BigDecimal queryForBigDecimal(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
		ResultSetReader<BigDecimal> resultSetReader = new ResultSetReader<BigDecimal>() {

			@Override
			public BigDecimal read(final ResultSet resultSet) throws SQLException {
				if ( resultSet.next() ) {
					BigDecimal result = resultSet.getBigDecimal(1) ;
					if (resultSet.next()) {
						throw new OrmNotUniqueResultException("The query execution returned a number of rows higher than 1");
					}
					return result;
				}
				throw new OrmNotUniqueResultException("The query execution has returned zero rows. One row was expected");
			}
		};
		return this.query(sql, resultSetReader, values);
	}

	@Override
	public final Object[] queryForArray(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
		ResultSetReader<Object[]> resultSetReader = new ResultSetReader<Object[]>() {

			@Override
			public Object[] read(final ResultSet resultSet) throws SQLException {
				int columnNumber = resultSet.getMetaData().getColumnCount();
				if ( resultSet.next() ) {
					Object[] result = new Object[columnNumber];
					for (int i=0 ; i<columnNumber ; i++) {
						result[i] = resultSet.getObject(i+1);
					}
					if (resultSet.next()) {
						throw new OrmNotUniqueResultException("The query execution returned a number of rows higher than 1");
					}
					return result;
				}
				throw new OrmNotUniqueResultException("The query execution has returned zero rows. One row was expected");
			}
		};
		return this.query(sql, resultSetReader, values);
	}

	@Override
	public final List<Object[]> queryForList(final String sql, final Object... values) throws OrmException {
		ResultSetReader<List<Object[]>> resultSetReader = new ResultSetReader<List<Object[]>>() {

			@Override
			public List<Object[]> read(final ResultSet resultSet) throws SQLException {
				int columnNumber = resultSet.getMetaData().getColumnCount();
				List<Object[]> resultList = new ArrayList<Object[]>();
				while ( resultSet.next() ) {
					Object[] objectArray = new Object[columnNumber];
					for (int i=0 ; i<columnNumber ; i++) {
						objectArray[i] = resultSet.getObject(i+1);
					}
					resultList.add(objectArray);
				}
				return resultList;
			}
		};
		return this.query(sql, resultSetReader, values);
	}

	@Override
	public void execute(final String sql) throws OrmException {
		this.logger.debug("Execute query: [" + sql + "]");
		this.sqlPerformerTemplate.execute(sql, this.getQueryTimeout());
	}

	@Override
	public <T> T query(final String sql, final ResultSetReader<T> rse, final Object... args) throws OrmException {
		this.logger.debug("Execute query: [" + sql + "]");
		if (args!=null) {
			this.logger.debug("Query params: " + Arrays.toString(args) );
		}		SqlSelectQuery query = new SqlSelectQuery(sql, args, this.getMaxRows(), this.getQueryTimeout());
		return this.sqlPerformerTemplate.query(query, rse);
	}

	@Override
	public int update(final String sql, final Object... args) throws OrmException {
		this.logger.debug("Execute query: [" + sql + "]");
		if (args!=null) {
			this.logger.debug("Query params: " + Arrays.toString(args));
		}		return this.sqlPerformerTemplate.update(sql, this.getQueryTimeout(), args);
	}

	@Override
	public int update(final String sql, final GeneratedKeyReader generatedKeyReader, final Object... args) throws OrmException {
		this.logger.debug("Execute query: [" + sql + "]");
		if (args!=null) {
			this.logger.debug("Query params: " + Arrays.toString(args) );
		}
		return this.sqlPerformerTemplate.update(sql, this.getQueryTimeout(), generatedKeyReader, args);
	}

	@Override
	public int[] batchUpdate(final List<String> sqls) throws OrmException {
		return this.sqlPerformerTemplate.batchUpdate(sqls, this.getQueryTimeout());
	}

	@Override
	public int[] batchUpdate(final String sql, final List<Object[]> args) throws OrmException {
		this.logger.debug("Execute query: [" + sql + "]");
		return this.sqlPerformerTemplate.batchUpdate(sql, args, this.getQueryTimeout());
	}

	@Override
	public int[] batchUpdate(final String sql, final PreparedStatementCreator psc) throws OrmException {
		this.logger.debug("Execute query: [" + sql + "]");
		return this.sqlPerformerTemplate.batchUpdate(sql, psc, this.getQueryTimeout());
	}

}
